	****************************************************************************
	* Tibeb analysis and tables
	*                              									 
	* Do-file 														 				
	* Written by: 		Menaal Ebrahim				     		 	                          			 
	* Status: 			Work in Progress            			 
	* First created:				 
	****************************************************************************				 
	
	****************************************************************************
	use "$data/elearning_bl_fu_indicators.dta", clear
	***************************************************************************
	
	****************************************************************************
	*Define globals:
	****************************************************************************
	
	global chars 			age_bl post_secondary children_bl hhsize_bl head_bl w1_hh_income_bl ///
							w1_businesses_owned_bl digital_skills_bl services_bl trade_bl years_operation_bl	
							
	global primary_bl		open_bl w5_mainbus_profits_bl w5_mainbus_revenues_bl 
	
	global secondary_bl		w1_mainbus_employees_bl w1_mainbus_hours_bl w1_mainbus_openhours_bl w1_mainbus_assets_bl ///
							mainbus_investment_bl buspractice_score_bl
	
	global balance			$chars $primary_bl $secondary_bl
	
	global takeup			course_attend course_attend_complete coursesstarted partial_full_complete courses_sessions_completed courses_comp_selfrep
	
	global surival			open own_business
	
	global profits			z_profits w5_mainbus_profits ihs_mainbus_profits w5_allbus_profits ihs_allbus_profits
							
	global sales 			w5_mainbus_revenues ihs_mainbus_revenues ihs_allbus_revenues 
							
	global primary_short	open z_profits w5_mainbus_profits w5_allbus_profits w5_mainbus_revenues w5_allbus_revenues
								
	global inputs			z_inputs w1_mainbus_employees w1_mainbus_hours w1_mainbus_openhours 
	
	global capital			z_capital w1_mainbus_assets ihs_mainbus_assets mainbus_investment w1_investment_amt ihs_investment_amt 
	
	global practices		buspractice_score knowledge_score
	
	global knowledge		kn3_corr kn4_corr kn5_corr kn6_corr kn7_corr kn8_corr
	
	global secondary_short 	z_inputs w1_mainbus_employees w1_mainbus_hours w1_mainbus_openhours ///
							z_capital w1_mainbus_assets mainbus_investment ///
							buspractice_score 
	
	//Predefine regression esttab options
	local esttaboptions  		replace stats(N mean p1_p2, labels("Observations" "Control mean" "P-val t1=t2") fmt(0 2 2)) booktabs b(2) se(2) eqlabels(none) label starlevels( * 0.10 ** 0.05 *** 0.01) substitute("0.0010" "0.01" "-0.00" "0.00") 
	
								

	//Program to combine and stack esttab tables
	cap prog drop panelcombine
	prog define panelcombine
	qui {
	syntax, use(str asis) paneltitles(str asis) columncount(integer) save(str asis) [CLEANup]
	preserve

	tokenize `"`paneltitles'"'
	//read in loop
	local num 1
	while "``num''"~="" {
	local panel`num'title="``num''"
	local num=`num'+1
	}


	tokenize `use'
	//read in loop
	local num 1
	while "``num''"~="" {
	tempfile temp`num'
	insheet using "``num''", clear
	save `temp`num''
	local max = `num'
	local num=`num'+1
	}

	//conditional processing loop
	local num 1
	while "``num''"~="" {
	local panellabel : word `num' of `c(ALPHA)'
	use `temp`num'', clear
		if `num'==1 { //process first panel -- clip bottom
		drop if strpos(v1,"Note:")>0 | strpos(v1,"in parentheses")>0 | strpos(v1,"p<0")>0
		drop if v1=="\end{tabular}" | v1=="}"
		replace v1 = "\hline \multicolumn{`columncount'}{l}{ \linebreak \underline{`panel1title'}} \\" if v1=="\hline" & _n<8 
		replace v1 = "\hline" if v1=="\hline\hline" & _n>4 //this is intended to replace the bottom double line; more robust condition probably exists
		}

	save `temp`num'', replace
	local num=`num'+1
	}

	use `temp1',clear
	local num 2
	while "``num''"~="" {
	append using `temp`num''
	local num=`num'+1
	}

	outsheet using `save', noname replace noquote


		if "`cleanup'"!="" { //erasure loop
		tokenize `use'
		local num 1
			while "``num''"~="" {
			erase "``num''"
			local num=`num'+1
			}
		}

	restore
	}
	end
	
	****************************************************************************
	
	
								*MAIN PAPER TABLES 
	
	
	
	****************************************************************************
	****************************************************************************
	*Table 1. Balance Check
	****************************************************************************

	local 	n: 											 list sizeof global(balance)
	local 	n=`n'
    mat full_sample_means = J(`n', 2, .)
    mat means1 = J(`n', 1, .)
	mat means2 = J(`n', 1, .)
	mat stars_t1 = J(`n', 1, 0)
	mat stars_t2 = J(`n', 1, 0)
	mat p_main = J(`n', 1, .)
	 
    mat rownames means1 = $balance 
 
    local y_counter = 1
    foreach var of global balance {
		//All 
		sum `var' 
        mat full_sample_means[`y_counter', 1] = `r(mean)'
        local N_all = `r(N)'
		
        // Control 
		sum `var' if treat4==1
        mat full_sample_means[`y_counter', 2] = `r(mean)'
        local N_control = `r(N)'
		
        sum `var' if treatpool1==1
        local N_T1 = `r(N)'
		
		sum `var' if treatpool2==1
        local N_T2 = `r(N)'

		reg `var' treatpool1 treatpool2, robust absorb(rand_round)
		mat means1[`y_counter', 1] = _b[treatpool1]
		local se = r(table)[2,1]
		matrix stars_t1[`y_counter', 1] = (abs(means1[`y_counter',1]/`se') > invttail(`e(df_r)',0.1/2)) + (abs(means1[`y_counter',1]/`se') > invttail(`e(df_r)',0.05/2)) + (abs(means1[`y_counter',1]/`se') > invttail(`e(df_r)',0.01/2))
		
		reg `var' treatpool1 treatpool2, robust absorb(rand_round)
		mat means2[`y_counter', 1] = _b[treatpool2]
		local se = r(table)[2,2]
		matrix stars_t2[`y_counter', 1] = (abs(means2[`y_counter',1]/`se') > invttail(`e(df_r)',0.1/2)) + (abs(means2[`y_counter',1]/`se') > invttail(`e(df_r)',0.05/2)) + (abs(means2[`y_counter',1]/`se') > invttail(`e(df_r)',0.01/2))

			
        test treatpool1=treatpool2
		local p = `r(p)'
        mat p_main[`y_counter', 1] = `p'
   
        local ++y_counter
    }

	//Joint orthogonality test
	reg treatpool1 $balance if treatpool2!=1, absorb(rand_round) robust
	testparm $balance 
	di `r(p)'
	local p_all_1 = round(`r(p)', 0.001)
	
	reg treatpool2 $balance if treatpool1!=1, absorb(rand_round) robust
	testparm $balance 	
	di `r(p)'
	local p_all_2 = round(`r(p)', 0.001)
	
	
	//Joint orthogonality test - with Randomization inference
	ritest treatpool1 e(F),  reps(1000) seed(202)  : reg treatpool1 $balance if treatpool2!=1, absorb(rand_round) robust
	local p_all_1_ri = r(p)[1,1]
		
	ritest treatpool2 e(F),  reps(1000) seed(202)  : reg treatpool2 $balance if treatpool1!=1, absorb(rand_round) robust
	local p_all_2_ri = r(p)[1,1]

    qui frmttable, statmat(full_sample_means) sdec(2) varlabels //install sg97_5
    qui frmttable, statmat(means1) sdec(2) annotate(stars_t1) asymbol(*,**,***) varlabels merge 
	qui frmttable, statmat(means2) sdec(2) annotate(stars_t2) asymbol(*,**,***) varlabels merge 
    qui frmttable, statmat(p_main) sdec(2) varlabels merge
  
    frmttable using "$tables/t", ///
	ctitle( ///
            "", "\textbf{All}", "\textbf{Control}", "\textbf{App (T1)}", "\textbf{In-person (T2)}", "\textbf{T1=T2}" \ ///
            "Control Variable", "Mean", "Mean", "Coefficient", "Coefficient", "P-val" \ ///
            "", "(1)", "(2)", "(3)", "(4)" , "(5)" ///
        ) ///
        tex ///
        fragment ///
        varlabels ///
        nocenter ///
		addrows("\hline" "" "" "" "" "" \ ///
				"Observations", "`N_all'", "`N_control'", "`N_T1'", "`N_T2'", "" \ ///
				"Joint orthogonality test", "", "", "`p_all_1'", "`p_all_2'", "" \ ///
				"Joint orthogonality test (RI)", "", "", "`p_all_1_ri'", "`p_all_2_ri'", "") ///
	    replace 

    filefilter "$tables/t.tex" "$tables/Table_1_balance_mat.tex", from("\BSend{tabular}\BS\BS") to("\BSend{tabular}") replace
    rm "$tables/t.tex"
	
	
	****************************************************************************
	*Table 2. Impacts on Training Completion
	****************************************************************************
	local 	j=												 1
	local 	i=												 1
	local 	a=												 1
	local 	num: 											 list sizeof global(takeup)
	display `num'
									
	foreach var in $takeup {	
	eststo	table`j'_`i': 					pdslasso `var' treatpool1 treatpool2 ///
											($chars $primary_bl $secondary_bl i.rand_round) , ///
											partial(i.rand_round) robust
	
	test treatpool1=treatpool2
	local rp : di %5.4f `r(p)'
	if `rp'==0 {
	     estadd scalar p1_p2 				0.01
	}
	else {
	estadd scalar p1_p2						`rp'
	}

	
	eststo table`j'_`i'

	eststo table`j'_`i'
	sum 									`var' if treatment_group_pooled==3
	estadd scalar mean=						r(mean)
	eststo	table`j'_`i'
			
			
	local 	i=								 `i'+1
	local	a=								 `a'+1
	}	
	
	
	
	esttab 	table`j'_* using "$tables\Table_2_Impacts_takeup.tex",  ///
			replace stats(N mean p1_p2, labels("Observations" "Control mean" "P-val t1=t2") fmt(0 2 2)) ///
			booktabs b(2) se(2) eqlabels(none) label starlevels( * 0.10 ** 0.05 *** 0.01)  ///
			mgroups("\textbf{Admin Data}" "\textbf{Self-reported}", pattern(1 0 0 0 0 1) ///
			prefix(\multicolumn{@span}{c}{) suffix(}) span erepeat(\cmidrule(lr){@span})) ///
			mtitles( "\makecell{\text{Partial compliance} \\  {(at least 1 course)}}" ///
					 "\makecell{\text{Full compliance} \\  {(at least 8 courses)}}" ///
					 "\makecell{\text{App modules started}  \\ {or Training sessions attended}}" ///
					 "\makecell{\text{App modules partially completed}  \\ {or Training sessions attended}}" ///
					 "\makecell{\text{App modules completed} \\ {or Training sessions attended}}" ///
					 "\makecell{\text{App modules completed} \\ {or Training sessions attended}}") ///
			nonotes keep(treat*) 

	est drop *
	
	****************************************************************************
	*Table 3. Impact on Primary and Secondary outcomes 
	****************************************************************************
	local 	j=												 1
	local 	i=												 1
	local 	a=												 1
	local 	num: 											 list sizeof global(primary_short)
	display `num'

	
	foreach var in $primary_short {	
	eststo	table`j'_`i': 					pdslasso `var'_fu treatpool1 treatpool2 ///	
											(`var'_bl $chars $primary_bl $secondary_bl i.rand_round) , ///
											partial(`var'_bl i.rand_round) robust
	
	test treatpool1=treatpool2
	estadd scalar p1_p2						r(p)
	eststo table`j'_`i'

	sum 									`var'_fu if treatment_group_pooled==3
	estadd scalar mean=						r(mean)
	eststo	table`j'_`i'
			
			
	local 	i=								 `i'+1
	local	a=								 `a'+1
	}	
	
	esttab 	table`j'_* using "$tables\Table_3_Impacts_primary.tex", `esttaboptions' compress ///
			mgroups("\textbf{Panel A: Impact on Primary outcomes}", pattern(1 0 0 0 0 0 0 0 0) ///
			prefix(\multicolumn{@span}{c}{) suffix(}) span erepeat(\cmidrule(lr){@span})) ///
			prehead("\begin{tabular}{l*{9}{c}{}} \hline\hline") ///
			mtitles( "\makecell{\text{Business was operating} \\ {at follow-up} \\ {(Yes 1; No 0)}}" ///
					 "\makecell{\text{Profits and sales} \\ {Index} \\ {(SD)}}" ///
					 "\makecell{\text{Last month's profit} \\ {Main business} \\ {(USD)}}" ///
					 "\makecell{\text{Last month's profit} \\ {Main and secondary business} \\ {(USD)}}" ///
					 "\makecell{\text{Last month's revenues}  \\ {Main business} \\ {(USD)}}" ///
					 "\makecell{\text{Last month's revenues}  \\ {Main and secondary business} \\ {(USD)}}") ///
					 nonotes keep(treat*) 

	est drop *
	

	local 	j=												 1
	local 	i=												 1
	local 	a=												 1
	local 	num: 											 list sizeof global(secondary_short)
	display `num'
									
	foreach var in $secondary_short {		
											
	eststo	table`j'_`i': 					pdslasso `var'_fu treatpool1 treatpool2 ///	
											(`var'_bl $chars $primary_bl $secondary_bl i.rand_round) , ///
											partial(`var'_bl i.rand_round) robust
	
	test treatpool1=treatpool2
	estadd scalar p1_p2						r(p)
	eststo table`j'_`i'
	
	sum 									`var'_fu if treatment_group_pooled==3
	estadd scalar mean=						r(mean)
	eststo	table`j'_`i'
			
			
	local 	i=								 `i'+1
	local	a=								 `a'+1
	}	
	
	eststo	table`j'_`i': 					pdslasso knowledge_score_fu treatpool1 treatpool2 ///	
											($chars $primary_bl $secondary_bl i.rand_round) , ///
											partial(i.rand_round) robust
	
	
	test treatpool1=treatpool2
	estadd scalar p1_p2						r(p)
	eststo table`j'_`i'
	
	sum 									knowledge_score_fu if treatment_group_pooled==3
	estadd scalar mean=						r(mean)
	eststo	table`j'_`i'

	
	esttab 	table`j'_* using "$tables\Table_4_Impacts_secondary.tex", `esttaboptions' fragment ///
			mgroups("\textbf{Panel B: Impact on Secondary outcomes}", pattern(1 0 0 0 0 0 0 0 0) ///
			prefix(\multicolumn{@span}{c}{) suffix(}) span erepeat(\cmidrule(lr){@span})) ///
			mtitles( "\makecell{\text{Inputs} \\ {Index} \\ {(SD)}}" /// 
					 "\makecell{\text{Number of} \\ {employees}}" ///
					 "\makecell{\text{Number of}  \\ {hours worked}} " ///
					 "\makecell{\text{Number of} \\ {hours open}}" ///
					 "\makecell{\text{Capital} \\ {Index} \\ {(SD)}}" ///
					 "\makecell{\text{Business assets}  \\ {(USD)}}" ///
					 "\makecell{\text{Made investments} \\  {(Yes 1; No 0)}}" ///
					 "\makecell{\text{Business practice score} \\  {(Proportion)}}" ///
					 "\makecell{\text{Knowledge score} \\  {(Proportion)}}" ) ///
					 prefoot("\hline") ///
					 postfoot("\hline\hline \end{tabular}") ///
					 nonotes keep(treat*)

					 
	cd "${tables}" 
	panelcombine, use(Table_3_Impacts_primary.tex Table_4_Impacts_secondary.tex) columncount(9) paneltitles("" "") save(Table_3_Impacts_all.tex) cleanup

	est drop *
	
	
	****************************************************************************
	*Table 4: Correlates of take-up
	****************************************************************************

	gen log_completed= log(courses_sessions_completed + 1)
	lab var log_completed "Courses completed (log transformed)"
	
	
	global controls			age_bl post_secondary married_bl children_5_bl hhsize_bl head_bl w1_hh_income_bl ///
							network_size_bl self_efficacy_bl hours_care_working_bl hours_care_bl mobility_bl internet_good_bl ///
							w1_businesses_owned_bl buspractice_score_bl digital_skills_bl services_bl ///
							trade_bl years_operation_bl time_work_bl preferred_elearning_bl 
							
	
	local 	j=												 1
	local 	k=												 1

	forval i=1/2 {
			eststo	table`j'_`k': 					reg log_completed $controls i.phase if treatment_group_pooled==`i', robust
			sum 									log_completed if treatment_group_pooled==`i'
			estadd scalar mean=						r(mean)
			eststo	table`j'_`k'

			local k= 								`k'+1
		
			lasso linear 							log_completed $controls i.phase if treatment_group_pooled==`i',  ///
													selection(cv, folds(5)) rseed(2028022558)
			lassocoef
			eststo	table`j'_`k': 					reg log_completed `e(allvars_sel)' if treatment_group_pooled==`i'
			sum 									log_completed if treatment_group_pooled==`i'
			estadd scalar mean=						r(mean)
			eststo	table`j'_`k'

			local k= 								`k'+1
		}


	esttab 	table`j'_* using "$tables\Table_4_Takeup_predict.tex",  ///
			replace stats(N mean, labels("Observations" "Mean") fmt(0 2)) ///
			booktabs b(2) se(2) eqlabels(none) label starlevels( * 0.10 ** 0.05 *** 0.01) substitute("0.0010" "0.01") ///
			mgroups("\textbf{App-based training}" "\textbf{In-person}", pattern(1 0 1 0 ) ///
			prefix(\multicolumn{@span}{c}{) suffix(}) span erepeat(\cmidrule(lr){@span})) ///
			mtitles( "\makecell{\text{No of courses completed} \\ {(Log-transformed)} \\ {(All variables)}}" ///
					 "\makecell{\text{No of courses completed} \\ {(Log-transformed)} \\ {(LASSO-selected)}}" ///
					 "\makecell{\text{No of courses completed} \\ {(Log-transformed)} \\ {(All variables)}}" ///
					 "\makecell{\text{No of courses completed} \\ {(Log-transformed)} \\ {(LASSO-selected)}}") ///			
			nonotes drop(1.phase _cons)

	est drop *
	

	****************************************************************************
	*FIGURE 1: Course Completion
	****************************************************************************
	twoway  (hist coursesstarted if treatpool1==1, percent fcolor(gray%20) lcolor(gray%80) barw(0.6)) ///
			(hist courses_sessions_completed if treatpool1==1, percent fcolor(gray%60) lcolor(gray%80)), ///
			graphregion(fcolor(white)) ///
			legend(order(1 "Started" 2 "Completed") pos(6) col(2)) ///
			xtitle(Modules, size(small)) xlabel(,labsize (small)) ///
			title("Panel A: App-based training", justification(left) color(black) span pos(12) size(medium)) ///
			xla(0(1)11) yla(0(10)50, angle(0)) ///
			saving("$tables/course_hist_app.gph", replace)
			graph export "$tables/Course_hist_app.png", replace
			
			hist courses_sessions_completed if treatpool2==1, percent fcolor(gray%60) lcolor(gray%80) ///
			graphregion(fcolor(white)) ///
			xtitle(Classes Attended, size(small)) xlabel(,labsize (small)) ///
			title("Panel B: In-person training", justification(left) color(black) span pos(12) size(medium)) ///
			xla(0(1)11) yla(0(10)50, angle(0)) ///
			saving("$tables/course_hist_inp.gph", replace)
			graph export "$tables/Course_hist_inp.png", replace
				
	graph combine "$tables/Course_hist_app.gph" "$tables/Course_hist_inp.gph"
	graph export  "$tables/Fig_1_Course_hist.png", replace
	rm "$tables/Course_hist_app.gph" 
	rm "$tables/Course_hist_inp.gph"
	rm "$tables/Course_hist_app.png"
	rm "$tables/Course_hist_inp.png"
