	****************************************************************************
	* Tibeb e learning cleaning	
	*                              									 
	* Do-file 														 				
	* Written by: 		Menaal Ebrahim				     		 	                          			 
	* Status: 			Work in Progress            			 
	* First created:				 
	****************************************************************************				  	
	local exrate 0.018

	****************************************************************************
	*GENERATE INDICATORS
	****************************************************************************
	use "$data/elearning_bl_fu_anon_clean.dta", clear
	
	//Re-label treatment variable
	lab define treat	 			1 "App-based training" ///
									2 "App + networking" ///
									3 "In-person training" ///
									4 "Control" 
	lab val treatment_group treat
	lab var treatment_group "Treatment assignment"
	
	//Generate treatment binaries 
	tabulate treatment_group, generate(treat)
	lab var treat1 "App-based training"
	lab var treat2 "App + networking"
	lab var treat3 "In-person training"
	lab var treat4 "Control"
	
	//Create pooled treatment group
	clonevar treatment_group_pooled = treatment_group
	replace treatment_group_pooled=1 if treatment_group==2
	replace treatment_group_pooled=2 if treatment_group==3
	replace treatment_group_pooled=3 if treatment_group==4
	lab define treat2	 			1 "App-based training (T1)" ///
									2 "In-person training (T2)" ///
									3 "Control"
	lab val treatment_group_pooled treat2
	
	tabulate treatment_group_pooled, generate(treatpool)
	lab var treatpool1 "App-based training (T1)"
	lab var treatpool2 "In-person training (T2)"
	lab var treatpool3 "Control"

	*CLEAN/RECODE NUMERICAL VARIABLES
	****************************************************************************
	ds, has(type long)
	foreach var of varlist `r(varlist)' {
		replace `var' = .a if `var'==77 | `var'==-77 //Not applicable
		replace `var' = .b if `var'==98 | `var'==-98 //Don't know
		replace `var' = .c if `var'==99 | `var'==-99 | `var' == -999 | `var'==88 | `var'==-88 //Refused 
	}
										
	*BASELINE BALANCE VARIABLES
	****************************************************************************
	//Age
	clonevar age_bl=s1_4
	lab var age_bl "Age of respondent (years)"
	*Impute for those that were missing
	qui sum age_bl
	replace age_bl= r(mean) if mi(age_bl)
	
	//Post-secondary education
	gen post_secondary=0 
	replace post_secondary=1 if s1_11a>7 & !mi(s1_11a)
	lab var post_secondary "Completed post-secondary education"
	clonevar post_secondary_bl=post_secondary
	
	//Has children
	clonevar children_bl=dm2_bl
	lab var children_bl "Has children"
	
	//Household size
	clonevar hhsize_bl=dm6_bl
	lab var hhsize_bl "Household size"
	replace hhsize_bl=. if hhsize_bl<0
	*Impute for those that were missing
	qui sum hhsize_bl
	replace hhsize_bl=r(mean) if mi(hhsize_bl)
	
	//Is head of the household
	gen head_bl=0 if !mi(dm5_bl)
	replace head_bl=1 if dm5_bl==1
	lab var head_bl "Head of her hh"

	//Household income
	clonevar hh_income_bl=dm8_bl
	lab var hh_income_bl "Last month's household income"
	
	//Number of businesses owned
	clonevar businesses_owned_bl=mb1_bl
	lab var businesses_owned_bl "Number of businesses owned"
	replace businesses_owned_bl=. if (businesses_owned_bl==98 | businesses_owned_bl==77)
	*Impute for those that were missing
	qui sum businesses_owned_bl
	replace businesses_owned_bl=r(mean) if mi(businesses_owned_bl)

	//Digital skills
	clonevar digital_skills_bl = nt5_bl
	replace digital_skills_bl = 0 if nt5_bl == 6
	lab var digital_skills_bl "Self-rated ability to install app on phone (range 0-5)"
	
	//Alternate measure for digital skills
	gen digital_skills_2_bl = 0 
	replace digital_skills_2_bl = 1 if nt4_2_bl==1 
	lab var digital_skills_2_bl "Uses phone for other educational apps"
	
	//Sector of operation
	gen services_bl=0 if !mi(mb8_bl)
	replace services_bl=1 if mb8_bl==4
	lab var services_bl "Main business is in the services sector"

	gen trade_bl=0 if !mi(mb8_bl)
	replace trade_bl=1 if mb8_bl==3
	lab var trade_bl "Main business is in the trade sector"
	
	//Years of operation
	gen years_operation_bl=0 if !mi(mb10_bl)
	replace years_operation_bl=1 if mb10_bl==4
	lab var years_operation_bl "Main business is over 5 years old"
			
	//Additional covariates
	****************************************************************************
	
	//Married
	gen married_bl = 0
	replace married_bl = 1 if dm1_bl==2
	lab var married_bl "Married"
	
	//Has children less than 5 years old
	gen children_5_bl = 0
	replace children_5_bl = 1 if inrange(dm3a_bl,1,5)
	lab var children_5_bl "Has children aged 5 or less"

	//Networks
	clonevar network_size_bl = nt2_bl
	qui sum nt2_bl
	replace network_size_bl =`r(mean)' if network_size_bl<0
	lab var network_size_bl "Network size"
	
	//Self-efficacy score
	egen self_efficacy_bl = rowtotal(nt6_bl - nt12_bl), missing
	lab var self_efficacy_bl "Entrepreneurial self-efficacy score"
	
	//Hours spent on care
	clonevar hours_care_working_bl = dm11_bl
	replace hours_care_working_bl=. if hours_care_working_bl<0
	qui sum hours_care_working_bl
	replace hours_care_working_bl = `r(mean)' if mi(hours_care_working_bl)
	lab var hours_care_working_bl "Hours spent on care while working"
	
	clonevar hours_care_bl = dm12_bl
	replace hours_care_bl=. if hours_care_bl<0
	qui sum hours_care_bl
	replace hours_care_bl = `r(mean)' if mi(hours_care_bl)
	lab var hours_care_bl "Hours spent on care outside work"

	//Mobility constraint
	gen mobility_bl = 0
	replace mobility_bl = 1 if dm13a_bl==1 & dm13b_bl==1
	lab var mobility_bl "Able to meet clients alone without permission (mobility)"
	
	//Internet connectivity
	gen internet_good_bl = 0 
	replace internet_good_bl = 1 if s1_10==1
	lab var internet_good_bl "Has very reliable internet access"

	//Interested in in-person
	recode s1_19 (2=0), gen(interest_inperson_bl)
	lab var interest_inperson_bl "Interested in in-person training"
	sum interest_inperson_bl
	replace interest_inperson_bl = `r(mean)' if mi(interest_inperson_bl)
	
	//Interested in app
	recode s1_20 (2=0), gen(interest_app_bl)
	lab var interest_app_bl "Interested in app training"
	sum interest_app_bl
	replace interest_app_bl = `r(mean)' if mi(interest_app_bl)
	
	//Interested in app over in-person
	gen preferred_elearning_bl=0 if !mi(s1_21)
	replace preferred_elearning_bl=1 if s1_21==2
	lab var preferred_elearning_bl "Respondent indicated they prefer only app training"
	sum preferred_elearning_bl
	replace preferred_elearning_bl = `r(mean)' if mi(preferred_elearning_bl)
	
	****************************************************************************
	*PRIMARY OUTCOMES
	****************************************************************************
	
	*TAKE-UP
	****************************************************************************	
	foreach var of varlist *_rate {
		replace `var'=. if `var'==0
		gen `var'p = 0 if treatpool1==1
		replace `var'p=1 if `var'>0.5 & !mi(`var')
		gen `var's = 0 if treatpool1==1
		replace `var's = 1 if `var'>0 & !mi(`var')

	}

	egen partial_full_complete = rowtotal(*_ratep)
	egen course_complete = rowtotal(*_comp)
	
	//App take-up - Admin data
	gen course_started=0 
	replace course_started=1 if coursesstarted>1 & !mi(coursesstarted)
	lab var course_started "Started at least 2 courses"
	
	gen course_completed=0 
	replace course_completed=1 if course_complete>0 & !mi(course_complete)
	lab var course_completed "Completed at least 1 course"
	
	gen eligible_diploma=0 
	replace eligible_diploma=1 if course_complete>=8 & !mi(course_complete)
	lab var eligible_diploma "Completed 8 or more courses"

	replace n_sessions="0" if n_sessions=="N/A"
	destring n_sessions, replace 
	replace n_sessions=0 if mi(n_sessions)
	lab var n_sessions "In-person classes attended"
	
	gen attended_inperson=0 
	replace attended_inperson=1 if n_sessions>0 
	lab var attended_inperson "Attended at least 1 in-person class"
	
	clonevar course_attend= course_completed
	replace course_attend= attended_inperson if treatment_group==3
	lab var course_attend "At least 1 courses/session completed"
	
	gen completed_inperson=0 
	replace completed_inperson=1 if n_sessions>=8 
	lab var completed_inperson "Completed in-person training (attended 8 or more classes)"
	
	egen course_attend_complete = rowmax(eligible_diploma completed_inperson)
	lab var course_attend_complete "Completed training course"
	
	replace coursesstarted=0 if mi(coursesstarted)
	replace coursesstarted= coursesstarted-1 if coursesstarted>0
	replace coursesstarted= n_sessions if treatment_group==3
	lab var coursesstarted "Number of courses started on e-learning app"
	
	replace partial_full_complete =n_sessions if treatment_group==3
	lab var partial_full_complete "App modules partially completed"
	
	replace coursescompleted=0 if mi(coursescompleted) 
	lab var coursescompleted "Number of courses completed on e-learning app"
	
	clonevar courses_sessions_completed= course_complete
	replace courses_sessions_completed= n_sessions if treatment_group==3
	lab var courses_sessions_completed "Number of courses/sessions completed"
	
	clonevar courses_comp_selfrep = in23_fu
	replace courses_comp_selfrep = int10_fu if !mi(int10_fu)
	replace courses_comp_selfrep = 0 if mi(in23_fu) & mi(int10_fu) & followup==1
	lab var courses_comp_selfrep "Number of courses/sessions completed - Self-reported"
	
	//Networking participation
	egen n_messages_posted = rowtotal(Messagesposted_l*) if treatment_group==2
	lab var n_messages_posted "Number of messages posted on network groups"
	
	gen networking_participate=0 if !mi(n_messages_posted)
	replace networking_participate=1 if n_messages_posted>0 & !mi(n_messages_posted)
	lab var networking_participate "Posted in network groups at least once"
	
	*SURVIVAL
	****************************************************************************
	
	//Main business currently open:
	*Baseline
	gen open_bl = 0 if !mi(mb7_bl)
	replace open_bl = 1 if mb7_bl==1
	lab var open_bl "Business was open at baseline"
	
	*Followup
	gen open_fu = 0 if !mi(mb6a_fu)
	replace open_fu = 1 if mb7_fu==1 
	lab var open_fu "Business was open at followup"
	
	//Onwer has at least one business
	*Baseline - everyone had a business
	 
	*Followup
	gen own_business_fu = 1 if !mi(mb6a_fu)
	replace own_business_fu = 0 if mb6a_fu==0 & mb6c_fu==0
	lab var own_business_fu "Owns at least one business "
	
	//New businesses started
	
	*Baseline
	gen oth_business_bl = 0 if mi(mb24_bl)
	replace oth_business_bl = 1 if mb24_bl>0 & mb24_bl<10
	lab var oth_business_bl "Has other businesses"
	
	*Followup
	gen oth_business_fu = 0 if !mi(mb6a_fu)
	replace oth_business_fu = 1 if mb6c!=0 
	lab var oth_business_fu "Has other businesses"
	
	gen open_new_business_fu= 0 if !mi(mb6a_fu)
	replace open_new_business_fu = 1 if mb6a==0 & mb6c!=0
	replace open_new_business_fu = 1 if mb6a==1 & mb6c!=0 & mb6d_fu>mb24_bl
	
	*BUSINESS PERFORMANCE
	****************************************************************************
	//Main business profits
	*Baseline
	clonevar mainbus_profits_bl=mb20_bl 
	replace mainbus_profits=0 if mi(mb20_bl) & baseline==1
	lab var mainbus_profits_bl "Main business monthly profits"
	
	*Followup	
	clonevar mainbus_profits_fu=mb20_fu 
	replace mainbus_profits_fu=0 if mi(mb20_fu) & followup==1
	lab var mainbus_profits_fu "Main business monthly profits"
	
	//All business profits 
	*Baseline
	egen allbus_profits_bl= rowtotal(mb20_bl mb32_bl), missing
	lab var allbus_profits_bl "Main and secondary business monthly profits"
	
	*Followup
	egen allbus_profits_fu= rowtotal(mb20_fu mb32_fu), missing 
	replace allbus_profits_fu=0 if mi(mb20_fu) & mi(mb32_fu) & followup==1
	lab var allbus_profits_fu "Main and secondary business monthly profits"
	
	//Main business revenues
	*Baseline 
	clonevar mainbus_revenues_bl=mb18_bl
	replace mainbus_revenues_bl=0 if mi(mb18_bl) & baseline==1
	lab var mainbus_revenues_bl "Main business monthly revenue"
	
	*Followup
	clonevar mainbus_revenues_fu=mb18_fu 
	replace mainbus_revenues_fu=0 if mi(mb18_fu) & followup==1
	lab var mainbus_revenues_fu "Main business monthly revenue"
	
	//All business revenues
	*Baseline 
	egen allbus_revenues_bl= rowtotal(mb18_bl mb30_bl), missing
	replace allbus_revenues_bl=0 if mi(mb18_bl) & baseline==1
	lab var allbus_revenues_bl "Main and secondary business monthly revenue"

	*Followup
	egen allbus_revenues_fu= rowtotal(mb18_fu mb30_fu), missing
	replace allbus_revenues_fu=0 if mi(mb18_fu) & followup==1
	replace allbus_revenues_fu=. if mi(mb18_fu) & followup==0
	lab var allbus_revenues_fu "Main and secondary business monthly revenue"
	
	****************************************************************************
	*SECONDARY OUTCOMES
	****************************************************************************
	
	*BUSINESS INPUTS 
	****************************************************************************
	//Number of employees
	*Baseline
	egen mainbus_employees_bl = rowtotal(mb15_bl mb15a_bl), missing
	replace mainbus_employees_bl=0 if mi(mb15_bl) & baseline==1
	lab var mainbus_employees_bl "Number of employees in main business"
	
	*Followup
	egen mainbus_employees_fu = rowtotal(mb15_fu mb15a_fu), missing
	replace mainbus_employees_fu=0 if mi(mb15_fu) & followup==1
	replace mainbus_employees_fu=. if mi(mb15_fu) & mi(mb15a_fu) & followup==0
	lab var mainbus_employees_fu "Number of employees in main business"
	
	//Number of hours spent on business
	*Baseline
	clonevar mainbus_hours_bl = mb16b_bl
	replace mainbus_hours_bl=0 if mi(mb16b_bl) & baseline==1
	replace mainbus_hours_bl=. if mainbus_hours_bl<0
	*Impute for those missing
	qui sum mainbus_hours_bl
	replace mainbus_hours_bl=r(mean) if mi(mainbus_hours_bl)
	lab var mainbus_hours_bl "Number of hours respondent spends on her business"
	
	*Followup
	clonevar mainbus_hours_fu = mb16b_fu
	replace mainbus_hours_fu=0 if mi(mb16b_fu) & followup==1
	lab var mainbus_hours_fu "Number of hours respondent spends on her business"
	
	//Number of hours business is open
	*Baseline 
	clonevar mainbus_openhours_bl = mb17a_bl
	replace mainbus_openhours_bl=0 if mi(mb17a_bl) & baseline==1
	replace mainbus_openhours_bl=. if mainbus_openhours_bl<0
	*Impute for those missing
	qui sum mainbus_openhours_bl
	replace mainbus_openhours_bl=r(mean) if mi(mainbus_openhours_bl)
	lab var mainbus_openhours_bl "Number of hours main business is open"
	
	*Followup
	clonevar mainbus_openhours_fu = mb17a_fu
	replace mainbus_openhours_fu=0 if mi(mb17a_fu) & followup==1
	lab var mainbus_openhours_fu "Number of hours main business is open"
	
	*CAPITAL
	****************************************************************************
	//Assets
	*Baseline
	clonevar mainbus_assets_bl = mb21_bl
	replace mainbus_assets_bl=0 if mi(mb21_bl) & baseline==1
	lab var mainbus_assets_bl "Value of assets in main business"

	*Followup
	clonevar mainbus_assets_fu = mb21_fu
	replace mainbus_assets_fu=0 if mi(mb21_fu) & followup==1
	lab var mainbus_assets_fu "Value of assets in main business"
		
	//Any capital investment
	*Baseline
	clonevar mainbus_investment_bl = mb45_bl
	lab var mainbus_investment_bl "Made a large investment to their business in the past 6 months "
	
	*Followup
	clonevar mainbus_investment_fu = mb45_fu 
	lab var mainbus_investment_fu "Made a large investment to their business in the past 6 months "
	
	//Capital investments in the past 6 months
	*Baseline - not captured
	
	*Followup
	clonevar investment_amt_fu = mb45a_fu
	replace investment_amt_fu=0 if mi(mb45a_fu) & followup==1
	lab var investment_amt_fu "Investments made in the past 6 months"
	
	*BUSINESS KNOWLEDGE AND PRACTICES
	****************************************************************************
	//Business practices
	*Baseline
	egen buspractice_score_bl= rowtotal(mb39_bl - mb44_bl mb46_bl mb47_bl), missing
	replace buspractice_score_bl = buspractice_score_bl/8
	lab var buspractice_score_bl "Fraction of business practices followed (over 8 practices)"
	
	*Followup
	egen buspractice_score_fu= rowtotal(mb39_fu - mb44_fu mb46_fu mb47_fu mb48 mb49 mb50), missing
	egen buspractice_n = rownonmiss(mb39_fu mb40_fu mb41_fu mb42_fu mb43_fu mb44_fu mb46_fu mb47_fu mb48 mb49 mb50)
	replace buspractice_score_fu = buspractice_score_fu/buspractice_n
	lab var buspractice_score_fu "Fraction of business practices followed (over 8 practices)"
	
	//Knowledge
	*Baseline - not captured
	
	*Followup
	gen kn3_corr= 0 if followup==1
	replace kn3_corr=1 if kn3_fu==5

	gen kn4_corr=0 if followup==1
	replace kn4_corr=1 if kn4_fu==1
	
	gen kn5_corr=0 if followup==1
	replace kn5_corr=1 if kn5_fu==1
	
	gen kn6_corr=0 if !mi(kn6)
	replace kn6_corr=1 if kn6_fu==1
	
	gen kn7_corr=0 if !mi(kn7)
	replace kn7_corr=1 if kn7_fu==1
	
	gen kn8_corr=0 if !mi(kn8)
	replace kn8_corr=1 if kn8_fu==1
	
	egen knowledge_score_fu = rowtotal(kn3_corr kn4_corr kn5_corr kn6_corr kn7_corr kn8_corr), missing
	egen knowledge_n = rownonmiss(kn3_corr kn4_corr kn5_corr kn6_corr kn7_corr kn8_corr)
	replace knowledge_score_fu = knowledge_score_fu/knowledge_n
	lab var knowledge_score_fu "Fraction of correct answers to knowledge questions (over 3 questions)"
	
	//Time spent
	*Baseline
	clonevar time_work_bl = dm10_bl
	lab var time_work_bl "Time spent working in a day (hours)"
	
	clonevar time_care_whilework_bl = dm11_bl
	lab var time_care_whilework_bl "Time spent on care while working (hours)"
	
	clonevar time_care_bl = dm12_bl
	lab var time_care_bl "Time spent on care outside work (hours)"
	
	*Endline
	clonevar time_work_fu = dm10_fu
	lab var time_work_fu "Time spent working in a day (hours)"
	
	clonevar time_care_whilework_fu = dm11_fu
	lab var time_care_whilework_fu "Time spent on care while working (hours)"
	
	clonevar time_care_fu = dm12_fu
	lab var time_care_fu "Time spent on care outside work (hours)"
	
	//Decision-making
	*Baseline - not measured 
	gen decision_maker_fu = 0 if !mi(dcm1_fu)
	replace decision_maker_fu = 1 if dcm1_fu==1
	lab var decision_maker_fu "Sole decision-maker in her enterprise"
	
	//Networks
	*Baseline
	egen discuss_ideas_bl = rowtotal(nt1_1_bl nt1_2_bl nt1_3_bl nt1_4_bl nt1_5_bl nt1_6_bl nt1_96_bl), missing
	lab var discuss_ideas_bl "Number of groups consulted for business ideas"
	
	*Followup
	egen discuss_ideas_fu = rowtotal(nt1_1_fu nt1_2_fu nt1_3_fu nt1_4_fu nt1_5_fu nt1_6_fu nt1_96_fu), missing
	lab var discuss_ideas_fu "Number of groups consulted for business ideas"
	
	egen networking_score_fu = rowtotal(nt15_fu nt16_fu nt17_fu nt18_fu), missing
	lab var networking_score_fu "Networking practices score (out of 4)"
	
	//Attriters
	gen attriters_IE=0 
	replace attriters_IE=1 if !mi(open_fu)
	lab var attriters_IE "Attiters at follow-up"

		
	****************************************************************************
	*PART 3. TRANSFORM VARIABLES
	****************************************************************************
	*FURTHER CLEAN LIKELY ERRONEOUS ENTRIES	
	local vars_mi_etb_bl	hh_income_bl mainbus_profits_bl allbus_profits_bl mainbus_revenues_bl allbus_revenues_bl mainbus_assets_bl ///
							mainbus_profits_fu allbus_profits_fu mainbus_revenues_fu allbus_revenues_fu mainbus_assets_fu investment_amt_fu 
	foreach var of local vars_mi_etb {
	    replace `var'=. if (`var'>-100 & `var'<=10 & !mi(`var') & `var'!=0) 
	}
	
	*IMPUTE WITH MEAN IF VARIABLES HAVE SPECIAL CODES - BASELINE VARIABLES
	local vars_mi_bl		hhsize_bl hh_income_bl businesses_owned_bl ///
							mainbus_profits_bl allbus_profits_bl mainbus_revenues_bl allbus_revenues_bl ///
							mainbus_employees_bl mainbus_hours_bl mainbus_openhours_bl mainbus_assets_bl ///
							time_care_bl time_work_bl time_care_whilework_bl digital_skills_bl
							
	foreach var of local vars_mi_bl {
		replace `var'=. if (`var'== -999 | `var'== -99 | `var'==-98 | `var'==-97 | `var'==-88 | `var'==-77 | `var'==99 | `var'==98 | `var'==88 | `var'==77 )
		qui sum `var'
	    replace `var'=r(mean) if `var'==. | `var'==.a | `var'==.b | `var'==.c
	}
	
	*CHANGE TO MISSING IF VARIABLES HAVE SPECIAL CODES - FOLLOWUP VARIABLES
	
	local vars_mi_fu		mainbus_profits_fu allbus_profits_fu mainbus_revenues_fu allbus_revenues_fu ///
							mainbus_employees_fu mainbus_hours_fu mainbus_openhours_fu mainbus_assets_fu investment_amt_fu 
							
	foreach var of local vars_mi_fu {
	    replace `var'=. if (`var'== -999 | `var'== -99 | `var'==-98 | `var'==-97 | `var'==-88 | `var'==-77 | `var'==99 | `var'==98 | `var'==88 | `var'==77)
	}
	
	
	*CONVERT TO USD
	****************************************************************************
	local usd 			hh_income_bl mainbus_profits_bl mainbus_profits_fu ///
						allbus_profits_bl allbus_profits_fu mainbus_revenues_bl mainbus_revenues_fu ///
						allbus_revenues_bl allbus_revenues_fu mainbus_assets_bl mainbus_assets_fu investment_amt_fu 
						
	foreach var of local usd {
		replace `var'= `var'*`exrate'		
		local lbl : variable label `var'
		lab var `var'  `"`lbl' (USD)"'

	}
	
	*WINSORIZE
	****************************************************************************
	//99%
	local numeric		hh_income_bl businesses_owned_bl ///
						mainbus_profits_bl mainbus_profits_fu allbus_profits_bl allbus_profits_fu ///
						mainbus_revenues_bl mainbus_revenues_fu allbus_revenues_bl allbus_revenues_fu ///
						mainbus_employees_bl mainbus_employees_fu mainbus_hours_bl mainbus_hours_fu ///
						mainbus_openhours_bl mainbus_openhours_fu mainbus_assets_bl mainbus_assets_fu investment_amt_fu 
						
	foreach var of local numeric {
		capture winsor `var', gen(w1_`var') p(0.01)
			if !_rc {
				local lbl : variable label `var'
				lab var w1_`var'  `"`lbl'"'
			}
	}

	//95%
	local numeric		hh_income_bl mainbus_profits_bl mainbus_profits_fu allbus_profits_bl allbus_profits_fu ///
						mainbus_revenues_bl mainbus_revenues_fu allbus_revenues_bl allbus_revenues_fu ///
						mainbus_assets_bl mainbus_assets_fu investment_amt_fu 
						
	foreach var of local numeric {
		capture winsor `var', gen(w5_`var') p(0.05)
			if !_rc {
				local lbl : variable label `var'
				lab var w5_`var'  `"`lbl'"'
					}
	}


	*IHS TRANSFORMATIONS
	****************************************************************************
	local ihs 			hh_income_bl mainbus_profits_bl mainbus_profits_fu ///
						allbus_profits_bl allbus_profits_fu mainbus_revenues_bl mainbus_revenues_fu ///
						allbus_revenues_bl allbus_revenues_fu mainbus_assets_bl mainbus_assets_fu investment_amt_fu 
						
	foreach var of local ihs {
		gen ihs_`var'= ln(`var' + ((`var'^2 +1)^0.5))
		
		local lbl : variable label `var'
		lab var ihs_`var'  `"`lbl' (IHS)"'

	}
	
	
	*CONSTRUCT INDICES
	****************************************************************************
	
	//Performance index, Input index, and Capital index
		foreach var in w1_mainbus_profits w1_allbus_profits w1_mainbus_revenues w1_allbus_revenues ihs_mainbus_profits ihs_allbus_profits ihs_mainbus_revenues ihs_allbus_revenues w1_mainbus_employees w1_mainbus_hours w1_mainbus_openhours w1_mainbus_assets ihs_mainbus_assets {
	foreach var2 in fu bl {
		sum `var'_`var2' if treat4==1, d
		gen z_`var'_`var2'=(`var'_`var2'-r(mean))/r(sd)
		  }
	}
	
	sum w1_investment_amt_fu if treat4==1, d
	gen z_w1_investment_amt_fu= (w1_investment_amt_fu-r(mean))/r(sd)
	sum ihs_investment_amt_fu if treat4==1, d
	gen z_ihs_investment_amt_fu= (ihs_investment_amt_fu-r(mean))/r(sd)
	

    foreach var2 in fu bl {
	egen z_profits_`var2'=rowmean(z_w1_mainbus_profits_`var2' z_w1_allbus_profits_`var2' z_w1_mainbus_revenues_`var2' z_w1_allbus_revenues_`var2' z_ihs_mainbus_profits_`var2' z_ihs_allbus_profits_`var2' z_ihs_mainbus_revenues_`var2' z_ihs_allbus_revenues_`var2')
	lab var z_profits_`var2' "Profits and sales index (SD)"
	egen z_inputs_`var2' = rowmean(z_w1_mainbus_employees_`var2' z_w1_mainbus_hours_`var2' z_w1_mainbus_openhours_`var2')
	lab var z_inputs_`var2' "Inputs index (SD)"
		}

	egen z_capital_bl = rowmean(z_w1_mainbus_assets_bl z_ihs_mainbus_assets_bl)
	lab var z_capital_bl "Capital index (SD)"
	
	egen z_capital_fu = rowmean(z_w1_mainbus_assets_fu z_ihs_mainbus_assets_fu z_w1_investment_amt_fu z_ihs_investment_amt_fu)
	lab var z_capital_fu "Capital index (SD)"
	
	*INTERACTION VARIABLES
	****************************************************************************
	
	//Services sector or not
	clonevar services = services_bl
	
	gen servicesxtreatpool1 = services_bl*treatpool1
	lab var servicesxtreatpool1 "Service sector x T1"
	gen servicesxtreatpool2 = services_bl*treatpool2
	lab var servicesxtreatpool2 "Service sector x T2"
	
	//Trade sector or not 
	clonevar trade = trade_bl
	
	gen tradextreatpool1 = trade_bl*treatpool1
	lab var tradextreatpool1 "Trade sector x T1"
	gen tradextreatpool2 = trade_bl*treatpool2
	lab var tradextreatpool2 "Trade sector x T2"

	//Baseline profits (above/below median)
	sum ihs_mainbus_profits_bl, d
	gen profitsmed = 0 
	replace profitsmed = 1 if ihs_mainbus_profits_bl >=`r(p50)' & !mi(ihs_mainbus_profits_bl)
	lab var profitsmed "Baseline profits were above the median (0/1)"
	
	gen profitsmedxtreatpool1 = profitsmed*treatpool1
	lab var profitsmedxtreatpool1 "Profits x T1"
	gen profitsmedxtreatpool2 = profitsmed*treatpool2
	lab var profitsmedxtreatpool2 "Profits x T2"

	//Digital skills score 
	sum digital_skills_bl, d
	gen digskillsmed = 0
	replace digskillsmed = 1 if digital_skills_bl >=`r(p50)' & !mi(digital_skills_bl)
	lab var digskillsmed "Digital skills were above the median (0/1)"

	gen digskillsmedxtreatpool1 = digskillsmed*treatpool1
	lab var digskillsmedxtreatpool1 "Digital skills x T1"
	gen digskillsmedxtreatpool2 = digskillsmed*treatpool2
	lab var digskillsmedxtreatpool2 "Digital skills x T2"
		
	//Completed post-secondary education
	gen post_secondaryxtreatpool1 = post_secondary*treatpool1
	lab var post_secondaryxtreatpool1 "Post secondary education x T1"
	gen post_secondaryxtreatpool2 = post_secondary*treatpool2
	lab var post_secondaryxtreatpool2 "Post secondary education x T2"
		
	
	****************************************************************************
	*PART 4. ANONYMISE AND PREPARE DATASET
	****************************************************************************
	keep 				psi_id phase treatment_group* rand_round attriters_IE treat* age_bl - eligible_diploma coursesstarted ///
						course_attend course_attend_complete courses_comp_selfrep n_sessions - buspractice_score_fu ///
						knowledge_score_fu - ihs_investment_amt_fu kn3_corr kn4_corr kn5_corr kn6_corr kn7_corr kn8_corr ///
						z_profits_fu z_profits_bl z_inputs_bl z_inputs_fu z_capital_bl z_capital_fu time* decision* ///
						mb39* mb40* mb41* mb42* mb43* mb44* mb45* mb46* mb47* *xtreat* w5* discuss_ideas_bl discuss_ideas_fu networking_score_fu ///
						services profitsmed digskillsmed post_secondary ///
						internet_good self_efficacy_bl digital_skills_2_bl digskillsmed ///
						married_bl children_5_bl network_size_bl self_efficacy_bl hours_care_working_bl hours_care_bl mobility_bl ///
						internet_good_bl interest_inperson_bl interest_app_bl preferred_elearning_bl location_bl 
	sort psi_id 
	save  "$data/elearning_bl_fu_indicators.dta", replace
	

	****************************************************************************
	*PART 5. PREPARE SCREENING DATASET
	****************************************************************************
	use "$data/application_data_anon.dta", clear
	merge 1:1 psi_id using "$data/elearning_bl_fu_indicators.dta", gen(app_match) keep(master match) keepusing(psi_id)
	
	//Drop ineligible
	drop if (mi(s1_18) | s1_18==2) & app_match==1
	
	lab define matches 1 "Eligible, not in IE" 3 "IE sample"
	lab val app_match matches
	
	//Fluent in english
	clonevar english_bl = s1_13
	replace english_bl = 0 if english_bl==2
	lab var english_bl "Is fluent in English"
	sum english_bl if app_match==3
	replace english_bl=r(mean) if app_match==3 & mi(english_bl)
	
	//Number of employees
	clonevar employees_n = s1_15
	lab var employees_n "Number of paid employees"
	sum employees_n if app_match==3
	replace employees_n=r(mean) if app_match==3 & mi(employees_n)
	
	//Managed business over 5 years
	gen managed_bus_5years= 0 if !mi(s1_16)
	replace managed_bus_5years = 1 if s1_16==4
	lab var managed_bus_5years "Has managed business for more than 5 years"
	sum managed_bus_5years if app_match==3
	replace managed_bus_5years=r(mean) if app_match==3 & mi(managed_bus_5years)
	sum managed_bus_5years if app_match==1
	replace managed_bus_5years=r(mean) if app_match==1 & mi(managed_bus_5years)
	
	//Age
	clonevar age_bl=s1_4
	lab var age_bl "Age"
		
	//Post-secondary education
	gen post_secondary=0 
	replace post_secondary=1 if s1_11a>7 & !mi(s1_11a)
	lab var post_secondary "Completed post-secondary education"
	
	//Internet connectivity
	gen internet_good_bl = 0 
	replace internet_good_bl = 1 if s1_10==1
	lab var internet_good_bl "Has very reliable internet access"

	//Interested in app over in-person
	gen preferred_elearning_bl=0 if !mi(s1_21)
	replace preferred_elearning_bl=1 if s1_21==2
	lab var preferred_elearning_bl "Indicated they prefer only app training"
	sum preferred_elearning_bl if app_match==3
	replace preferred_elearning_bl = r(mean) if app_match==3 & mi(preferred_elearning_bl)
	
	//Location far from information sessions
	gen location = 	1 
	replace location = 0  if s1_17b==3 | s1_17b==11
	lab var location "Lives close to the information session venue"

	save  "$data/elearning_bl_fu_indicators_screening.dta", replace
